import numpy as np
import pytest
from aerobulk import flux_noskin, flux_skin
from aerobulk.flux import VALID_ALGOS, VALID_ALGOS_SKIN


@pytest.mark.parametrize(
    "algo",
    VALID_ALGOS
    + [pytest.param("wrong", marks=pytest.mark.xfail(strict=True, raises=ValueError))],
)
def test_smoke_noskin(algo):
    # I am very confused about shape.
    # We want Nt to be the *slowest varying dimension*
    # For C-order arrays, would give (Nt, Ny, Nx)
    # For F-order arrays, would give (Nx, Ny, Nt)
    # I expected that the shape passed from Python would reverse when it hits Fortran,
    # but this did not happen.
    shape = (200, 200, 10)
    order = "F"
    sst = np.full(shape, 290.0, order=order)
    t_zt = np.full(shape, 280.0, order=order)
    hum_zt = np.full(shape, 0.001, order=order)
    u_zu = np.full(shape, 1.0, order=order)
    v_zu = np.full(shape, -1.0, order=order)
    slp = np.full(shape, 101000.0, order=order)
    flux_noskin(sst, t_zt, hum_zt, u_zu, v_zu, slp, algo=algo)


@pytest.mark.parametrize(
    "algo",
    VALID_ALGOS_SKIN
    + [
        pytest.param("wrong", marks=pytest.mark.xfail(strict=True, raises=ValueError)),
        pytest.param("ncar", marks=pytest.mark.xfail(strict=True, raises=ValueError)),
    ],
)
def test_smoke_skin(algo):
    # I am very confused about shape.
    # We want Nt to be the *slowest varying dimension*
    # For C-order arrays, would give (Nt, Ny, Nx)
    # For F-order arrays, would give (Nx, Ny, Nt)
    # I expected that the shape passed from Python would reverse when it hits Fortran,
    # but this did not happen.
    shape = (200, 200, 10)
    order = "F"
    sst = np.full(shape, 290.0, order=order)
    t_zt = np.full(shape, 280.0, order=order)
    hum_zt = np.full(shape, 0.001, order=order)
    u_zu = np.full(shape, 1.0, order=order)
    v_zu = np.full(shape, -1.0, order=order)
    rad_sw = np.full(shape, 0.0, order=order)
    rad_lw = np.full(shape, 350.0, order=order)
    slp = np.full(shape, 101000.0, order=order)
    flux_skin(sst, t_zt, hum_zt, u_zu, v_zu, rad_sw, rad_lw, slp, algo=algo)



import aerobulk.aerobulk.mod_aerobulk_wrap as aero

VALID_ALGOS = ["coare3p0", "coare3p6", "ecmwf", "ncar", "andreas"]
VALID_ALGOS_SKIN = ["coare3p0", "coare3p6", "ecmwf"]


def _check_algo(algo, valids):
    if algo not in valids:
        raise ValueError(f"Algorithm {algo} not valid. Choose from {valids}.")


def flux_noskin(
    sst, t_zt, hum_zt, u_zu, v_zu, slp=101000.0, algo="coare3p0", zt=10, zu=2, niter=1
):
    """Python wrapper for aerobulk without skin correction.
    !ATTENTION If input not provided in correct units, will crash.

    Parameters
    ----------
    sst : array_like
        Bulk sea surface temperature [K]
    t_zt : array_like
        Absolute air temperature at height zt [K]
    hum_zt : array_like
        air humidity at zt, can be given as:
            - specific humidity                   [kg/kg]
            - dew-point temperature                   [K]
            - relative humidity                       [%]
            => type should normally be recognized based on value range
    u_zu : array_like
        zonal wind speed at zu                    [m/s]
    v_zu : array_like
        meridional wind speed at zu               [m/s]
    slp : array_like, optional
        mean sea-level pressure                   [Pa] ~101000 Pa,
        by default 101000.0
    algo : str, optional
        Algorithm, can be one of: "coare3p0", "coare3p6", "ecmwf", "ncar", "andreas",
        by default "coare3p0"
    zt : int, optional
        height for temperature and spec. hum. of air           [m],
        by default 10
    zu : int, optional
        height for wind (10m = traditional anemometric height  [m],
        by default 2
    niter : int, optional
        Number of iteration steps used in the algorithm,
        by default 1

    Returns
    -------
    ql : array_like
        Latent heat flux                                     [W/m^2]
    qh : array_like
        Sensible heat flux                                   [W/m^2]
    taux : array_like
        zonal wind stress                                    [N/m^2]
    tauy : array_like
        meridional wind stress                                    [N/m^2]
    evap : array_like
        evaporation         [mm/s] aka [kg/m^2/s] (usually <0, as ocean loses water!)
    """
    _check_algo(algo, VALID_ALGOS)
    ql, qh, taux, tauy, evap = aero.mod_aerobulk_wrapper.aerobulk_model_noskin(
        algo, zt, zu, sst, t_zt, hum_zt, u_zu, v_zu, slp, niter
    )
    return ql, qh, taux, tauy, evap


def flux_skin(
    sst,
    t_zt,
    hum_zt,
    u_zu,
    v_zu,
    rad_sw,
    rad_lw,
    slp=101000.0,
    algo="coare3p0",
    zt=10,
    zu=2,
    niter=1,
):
    """Python wrapper for aerobulk with skin correction.
    !ATTENTION If input not provided in correct units, will crash.

    Parameters
    ----------
    sst : array_like
        Bulk sea surface temperature [K]
    t_zt : array_like
        Absolute air temperature at height zt [K]
    hum_zt : array_like
        air humidity at zt, can be given as:
            - specific humidity                   [kg/kg]
            - dew-point temperature                   [K]
            - relative humidity                       [%]
            => type should normally be recognized based on value range
    u_zu : array_like
        zonal wind speed at zu                    [m/s]
    v_zu : array_like
        meridional wind speed at zu               [m/s]
    rad_sw : array_like
        downwelling shortwave radiation at the surface (>0)   [W/m^2]
    rad_lw : array_like
        rad_lw : downwelling longwave radiation at the surface  (>0)   [W/m^2]
    slp : array_like, optional
        mean sea-level pressure                   [Pa] ~101000 Pa,
        by default 101000.0
    algo : str, optional
        Algorithm, can be one of: "coare3p0", "coare3p6", "ecmwf",
        by default "coare3p0"
    zt : int, optional
        height for temperature and spec. hum. of air           [m],
        by default 10
    zu : int, optional
        height for wind (10m = traditional anemometric height  [m],
        by default 2
    niter : int, optional
        Number of iteration steps used in the algorithm,
        by default 1

    Returns
    -------
    ql : array_like
        Latent heat flux                                     [W/m^2]
    qh : array_like
        Sensible heat flux                                   [W/m^2]
    taux : array_like
        zonal wind stress                                    [N/m^2]
    tauy : array_like
        meridional wind stress                                    [N/m^2]
    t_s : array_like
        skin temperature    [K]    (only when l_use_skin_schemes=TRUE)
    evap : array_like
        evaporation         [mm/s] aka [kg/m^2/s] (usually <0, as ocean loses water!)
    """
    _check_algo(algo, VALID_ALGOS_SKIN)

    ql, qh, taux, tauy, t_s, evap = aero.mod_aerobulk_wrapper.aerobulk_model_skin(
        algo, zt, zu, sst, t_zt, hum_zt, u_zu, v_zu, slp, rad_sw, rad_lw, niter
    )
    return ql, qh, taux, tauy, t_s, evap



import numpy as np
import pytest
from aerobulk import flux_noskin, flux_skin


@pytest.mark.parametrize(
    "algo, expected",
    [
        (
            "andreas",
            {
                "ql": -74.4637756,
                "qh": -14.4129944,
                "evap": -2.62721038,
                "taux": 3.02770734e-02,
                "tauy": 0.0,
            },
        ),
        (
            "ncar",
            {
                "ql": -88.4781876,
                "qh": -16.6969528,
                "evap": -3.12166286,
                "taux": 3.58519591e-02,
                "tauy": 0.0,
            },
        ),
    ],
)
def test_fortran_noskin(algo, expected):
    # Compare python wrapper output with fortran results from (...)

    # inputs are the same for all test cases in the fortran example
    rt0 = 273.15  # conversion to Kelvin
    sst = rt0 + 22  # in Kelvin
    t_zt = rt0 + 20
    hum_zt = 0.012  # kg/kg
    u_zu = 5  # m/s
    v_zu = 0
    slp = 101000.0  # Pa
    ql, qh, taux, tauy, evap = flux_noskin(
        sst, t_zt, hum_zt, u_zu, v_zu, slp, algo=algo, zt=2, zu=10, niter=10
    )
    evap = evap * 3600 * 24  # convert to mm/day

    np.testing.assert_allclose(ql, expected["ql"])
    np.testing.assert_allclose(qh, expected["qh"])
    np.testing.assert_allclose(evap, expected["evap"])
    np.testing.assert_allclose(taux, expected["taux"])
    np.testing.assert_allclose(tauy, expected["tauy"])


@pytest.mark.parametrize(
    "algo, expected",
    [
        (
            "coare3p0",
            {
                "ql": -81.3846741,
                "qh": -15.1545086,
                "evap": -2.87061906,
                "taux": 3.57834995e-02,
                "tauy": 0.0,
                "t_s": 21.7219677,
            },
        ),
        (
            "coare3p6",
            {
                "ql": -83.0788422,
                "qh": -15.3865499,
                "evap": -2.93033028,
                "taux": 3.21817845e-02,
                "tauy": 0.0,
                "t_s": 21.7057972,
            },
        ),
        (
            "ecmwf",
            {
                "ql": -80.2958984,
                "qh": -14.3822346,
                "evap": -2.83224440,
                "taux": 3.84389125e-02,
                "tauy": 0.0,
                "t_s": 21.7325401,
            },
        ),
    ],
)
def test_fortran_skin(algo, expected):
    # Compare python wrapper output with fortran results from (...)

    # inputs are the same for all test cases in the fortran example
    rt0 = 273.15  # conversion to Kelvin
    sst = rt0 + 22  # in Kelvin
    t_zt = rt0 + 20
    hum_zt = 0.012  # kg/kg
    u_zu = 5  # m/s
    v_zu = 0
    slp = 101000.0  # Pa
    rad_sw = 0
    rad_lw = 350
    ql, qh, taux, tauy, t_s, evap = flux_skin(
        sst,
        t_zt,
        hum_zt,
        u_zu,
        v_zu,
        rad_sw,
        rad_lw,
        slp,
        algo=algo,
        zt=2,
        zu=10,
        niter=10,
    )
    evap = evap * 3600 * 24  # convert to mm/day
    t_s = t_s - rt0

    np.testing.assert_allclose(ql, expected["ql"])
    np.testing.assert_allclose(qh, expected["qh"])
    np.testing.assert_allclose(evap, expected["evap"])
    np.testing.assert_allclose(taux, expected["taux"])
    np.testing.assert_allclose(tauy, expected["tauy"])
    np.testing.assert_allclose(t_s, expected["t_s"])



from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("aerobulk-python")
except PackageNotFoundError:
    __version__ = "unknown"
    pass

from .flux import flux_noskin, flux_skin



import os

# We need to import setuptools here in order for it to persist in sys.modules.
# Its presence/absence is used in subclassing setup in numpy/distutils/core.py.
# However, we need to run the distutils version of sdist, so import that first
# so that it is in sys.modules
import numpy.distutils.command.sdist  # noqa
import setuptools  # noqa
from numpy.distutils.core import Extension, setup
from numpy.distutils.fcompiler import get_default_fcompiler

# Trying this from the numpy setup.py


here = os.path.dirname(__file__)
with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

install_requires = [
    "numpy",
]

# figure out which compiler we're going to use
compiler = get_default_fcompiler()
# set some fortran compiler-dependent flags
f90flags = []
if compiler == "gnu95":
    # f90flags.append('-fno-range-check')
    # f90flags.append('-ffree-form')
    # These are the flags I used in the manual version (might have some more here)
    f90flags.append("-fdefault-real-8")
    f90flags.append("-ffree-line-length-200")
elif compiler == "intel" or compiler == "intelem":
    f90flags.append("-132")
#  Set aggressive optimization level
f90flags.append("-O3")
#  Suppress all compiler warnings (avoid huge CI log files)
f90flags.append("-w")

# for this API we will only expose a single extension?
ext_modules = [
    Extension(
        name="mod_aerobulk_wrap",
        sources=[
            "./source/fortran/aerobulk/src/mod_const.f90",
            "./source/fortran/aerobulk/src/mod_phymbl.f90",
            "./source/fortran/aerobulk/src/mod_skin_coare.f90",
            "./source/fortran/aerobulk/src/mod_skin_ecmwf.f90",
            "./source/fortran/aerobulk/src/mod_blk_andreas.f90",
            "./source/fortran/aerobulk/src/mod_common_coare.f90",
            "./source/fortran/aerobulk/src/mod_blk_coare3p0.f90",
            "./source/fortran/aerobulk/src/mod_blk_coare3p6.f90",
            "./source/fortran/aerobulk/src/mod_blk_ecmwf.f90",
            "./source/fortran/aerobulk/src/mod_blk_ncar.f90",
            "./source/fortran/aerobulk/src/mod_blk_neutral_10m.f90",
            "./source/fortran/aerobulk/src/mod_aerobulk_compute.f90",
            "./source/fortran/aerobulk/src/mod_aerobulk.f90",
            "./source/fortran/mod_aerobulk_wrap.f90",
            "./source/fortran/mod_aerobulk_wrap.pyf",
        ],
        extra_f90_compile_args=f90flags,
        # f2py_options=['--quiet'],
    )
]

setup(
    name="aerobulk-python",
    description="General Circulation Model Postprocessing with xarray",
    url="https://github.com/xgcm/aerobulk-python",
    author="aerobulk-python Developers",
    author_email="julius@ldeo.columbia.edu",
    license="GPLv3",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    install_requires=install_requires,
    python_requires=">=3.8",
    # long_description=long_description,
    # long_description_content_type="text/x-rst",
    setup_requires="setuptools_scm",
    use_scm_version={
        "write_to": "source/aerobulk/_version.py",
        "write_to_template": '__version__ = "{version}"',
        "tag_regex": r"^(?P<prefix>v)?(?P<version>[^\+]+)(?P<suffix>.*)?$",
    },
    package_dir={"": "source"},
    packages=["aerobulk"],
    ext_package="aerobulk.aerobulk",
    ext_modules=ext_modules,
)
